"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var spsave_1 = require("spsave");
var spauth = require("node-sp-auth");
var sprequest = require("sp-request");
var ReloadProvisioning = (function () {
    function ReloadProvisioning(settings) {
        this.ctx = __assign({}, settings, { port: typeof settings.port !== 'undefined' ? settings.port : 3000, host: typeof settings.host !== 'undefined' ? settings.host : 'localhost', protocol: typeof settings.protocol !== 'undefined' ? settings.protocol :
                (settings.siteUrl || '').toLowerCase().indexOf('https://') === 0 ? 'https' : 'http' });
        this.spr = this.getCachedRequest();
    }
    ReloadProvisioning.prototype.getUserCustomActions = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.spr = _this.getCachedRequest();
            _this.spr.get(_this.ctx.siteUrl + "/_api/site/usercustomactions")
                .then(function (response) {
                resolve(response.body.d.results);
            });
        });
    };
    ReloadProvisioning.prototype.getSiteData = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.spr = _this.getCachedRequest();
            _this.spr.get(_this.ctx.siteUrl + "/_api/site")
                .then(function (response) {
                resolve(response.body.d);
            });
        });
    };
    ReloadProvisioning.prototype.provisionMonitoringAction = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var devBaseUrl = (_this.ctx.protocol + "://" + _this.ctx.host + ":" + _this.ctx.port)
                .replace(':80', '').replace(':443', '');
            _this.getSiteData()
                .then(function (data) {
                return _this.deployClientScript(data.Url);
            })
                .then(function () {
                return _this.getUserCustomActions();
            })
                .then(function (customActions) {
                var cas = customActions.filter(function (ca) {
                    return ca.Title === 'LiveReloadCustomAction';
                });
                if (cas.length === 0) {
                    resolve(_this.provisionCustomAction());
                }
                else {
                    reject({
                        message: 'Warning: Live Reload custom action has already been deployed. Skipped.'
                    });
                }
            });
        });
    };
    ReloadProvisioning.prototype.retractMonitoringAction = function () {
        var _this = this;
        return this.getUserCustomActions()
            .then(function (customActions) {
            customActions.forEach(function (ca) {
                if (ca.Title === 'LiveReloadCustomAction') {
                    return _this.deleteCustomAction(ca.Id);
                }
            });
        });
    };
    ReloadProvisioning.prototype.deployClientScript = function (siteCollectionUrl) {
        var fileContent = String(fs.readFileSync(path.join(__dirname, '/../', '/static/live-reload.client.js')));
        fileContent = fileContent.replace('"##settings#"', JSON.stringify({
            protocol: this.ctx.protocol,
            host: this.ctx.host,
            port: this.ctx.port
        }));
        var core = {
            siteUrl: siteCollectionUrl || this.ctx.siteUrl,
            flatten: false,
            checkin: true,
            checkinType: 1
        };
        var fileOptions = {
            fileName: 'live-reload.client.js',
            fileContent: fileContent,
            folder: '_catalogs/masterpage/spf/dev'
        };
        return spsave_1.spsave(core, this.ctx.creds, fileOptions);
    };
    ReloadProvisioning.prototype.provisionCustomAction = function () {
        var _this = this;
        this.spr = this.getCachedRequest();
        var reqBody = {
            '__metadata': {
                'type': 'SP.UserCustomAction'
            },
            'Title': 'LiveReloadCustomAction',
            'Location': 'ScriptLink',
            'Description': 'Live Reload Custom Action',
            'ScriptSrc': '~sitecollection/_catalogs/masterpage/spf/dev/live-reload.client.js',
            'Sequence': '10000'
        };
        return this.spr.requestDigest(this.ctx.siteUrl)
            .then(function (digest) {
            return _this.spr.post(_this.ctx.siteUrl + "/_api/site/usercustomactions", {
                headers: {
                    'X-RequestDigest': digest,
                    'Accept': 'application/json; odata=verbose',
                    'Content-Type': 'application/json; odata=verbose'
                },
                body: reqBody
            });
        });
    };
    ReloadProvisioning.prototype.deleteCustomAction = function (customActionId) {
        var _this = this;
        this.spr = this.getCachedRequest();
        return this.spr.requestDigest(this.ctx.siteUrl)
            .then(function (digest) {
            return _this.spr.post(_this.ctx.siteUrl + "/_api/site/usercustomactions('" + customActionId + "')", {
                headers: {
                    'X-RequestDigest': digest,
                    'X-HTTP-Method': 'DELETE'
                }
            });
        })
            .then(function (response) {
            return response.body;
        });
    };
    ReloadProvisioning.prototype.getAuthOptions = function () {
        return spauth.getAuth(this.ctx.siteUrl, this.ctx.creds);
    };
    ReloadProvisioning.prototype.getCachedRequest = function () {
        return this.spr || sprequest.create(this.ctx.creds);
    };
    return ReloadProvisioning;
}());
exports.ReloadProvisioning = ReloadProvisioning;
